from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.error_response import ErrorResponse
from ...models.organization_invitation import OrganizationInvitation
from ...types import Response


def _get_kwargs(
    org_name: str,
) -> Dict[str, Any]:
    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": f"/v1/organizations/{org_name}/invitations",
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[ErrorResponse, List["OrganizationInvitation"]]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = OrganizationInvitation.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = ErrorResponse.from_dict(response.json())

        return response_400
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = ErrorResponse.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = ErrorResponse.from_dict(response.json())

        return response_404
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[ErrorResponse, List["OrganizationInvitation"]]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    org_name: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Union[ErrorResponse, List["OrganizationInvitation"]]]:
    """Get all invitations for an organization

     This endpoint allows an organization admin to get all invitations for their organization.

    Args:
    ----
        org_name (str):

    Raises:
    ------
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
    -------
        Response[Union[ErrorResponse, List['OrganizationInvitation']]]

    """
    kwargs = _get_kwargs(
        org_name=org_name,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    org_name: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[Union[ErrorResponse, List["OrganizationInvitation"]]]:
    """Get all invitations for an organization

     This endpoint allows an organization admin to get all invitations for their organization.

    Args:
    ----
        org_name (str):

    Raises:
    ------
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
    -------
        Union[ErrorResponse, List['OrganizationInvitation']]

    """
    return sync_detailed(
        org_name=org_name,
        client=client,
    ).parsed


async def asyncio_detailed(
    org_name: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Union[ErrorResponse, List["OrganizationInvitation"]]]:
    """Get all invitations for an organization

     This endpoint allows an organization admin to get all invitations for their organization.

    Args:
    ----
        org_name (str):

    Raises:
    ------
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
    -------
        Response[Union[ErrorResponse, List['OrganizationInvitation']]]

    """
    kwargs = _get_kwargs(
        org_name=org_name,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    org_name: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[Union[ErrorResponse, List["OrganizationInvitation"]]]:
    """Get all invitations for an organization

     This endpoint allows an organization admin to get all invitations for their organization.

    Args:
    ----
        org_name (str):

    Raises:
    ------
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
    -------
        Union[ErrorResponse, List['OrganizationInvitation']]

    """
    return (
        await asyncio_detailed(
            org_name=org_name,
            client=client,
        )
    ).parsed
